        ;
        ; Epilogue for IntyBASIC programs
        ; by Oscar Toledo G.  http://nanochess.org/
        ;
        ; Revision: Jan/30/2014. Moved GRAM code below MOB updates.
        ;                        Added comments.

        ;
        ; Wait for interruption
        ;
_wait:  PROC
        CLRR    R0
        MVO     R0,_int         ; Clears waiting flag
@@1:    CMP     _int,  R0       ; Waits for change
        BEQ     @@1
        JR      R5              ; Returns
        ENDP

        ;
        ; Interruption routine
        ;
_int_vector:     PROC
        BEGIN

        MVO     R0,     $20     ; Activates display
        MVI     $21,    R0      ; Activates mode for 64 definitions of MOB
        MVII    #1,     R0
        MVO     R0,     _int    ; Indicates interrupt happened
        MVI _first_time,R0
        TSTR R0
        BEQ @@vi0
        DECR R0
        MVO R0,_first_time
        MVII    #0,     R0
        MVO     R0,     $28     ; Color stack 0
        MVO     R0,     $2A     ; Color stack 2
        MVO     R0,     $2C     ; Border color
@@vi0:
        ;
        ; Save collision registers for further use
        ;
        MVI     $18,R0
        MVO     R0,_col0
        MVI     $19,R0
        MVO     R0,_col1
        MVI     $1A,R0
        MVO     R0,_col2
        MVI     $1B,R0
        MVO     R0,_col3
        MVI     $1C,R0
        MVO     R0,_col4
        MVI     $1D,R0
        MVO     R0,_col5
        MVI     $1E,R0
        MVO     R0,_col6
        MVI     $1F,R0
        MVO     R0,_col7
        CLRR    R0
        MVO 	R0,$18
        MVO 	R0,$19
        CLRR    R0
        MVO     R0,$1A
        MVO     R0,$1B
        CLRR    R0
        MVO     R0,$1C
        MVO     R0,$1D
        CLRR    R0
        MVO 	R0,$1E
        MVO     R0,$1F

        ;
        ; Updates sprites (MOBs)
        ;
        MVII #_mobs,R4
        MVII #$0,R5     ; X-coordinates
        MVII #8,R1
@@vi2:  MVI@ R4,R0
        MVO@ R0,R5
        MVI@ R4,R0
        MVO@ R0,R5
        MVI@ R4,R0
        MVO@ R0,R5
        DECR R1
        BNE @@vi2

        ;
        ; Detect GRAM definition
        ;
        MVI _load_gram,R0
        TSTR R0
        BEQ @@vi1
        CLRR R0
        MVO R0,_load_gram
        MVI _gram_bitmap,R4
        MVI _gram_target,R1
        SLL R1,2
        SLL R1,1
        ADDI #$3800,R1
        MOVR R1,R5
        MVI _gram_total,R0
@@vi3:
        MVI@    R4,     R1
        MVO@    R1,     R5
        SWAP    R1
        MVO@    R1,     R5
        MVI@    R4,     R1
        MVO@    R1,     R5
        SWAP    R1
        MVO@    R1,     R5
        MVI@    R4,     R1
        MVO@    R1,     R5
        SWAP    R1
        MVO@    R1,     R5
        MVI@    R4,     R1
        MVO@    R1,     R5
        SWAP    R1
        MVO@    R1,     R5
        DECR R0
        BNE @@vi3
@@vi1:
        ;
        ; Random number generator
        ;
MACRO _ROR
	RRC R0,1
	MOVR R0,R1
	SLR R1,2
	SLR R1,2
	ANDI #$0800,R1
	SLR R1,2
	SLR R1,2
	ANDI #$007F,R0
	XORR R1,R0
ENDM
        MVI _rand,R0
        SETC
        _ROR
        XOR _frame,R0
        _ROR
        XOR _rand,R0
        _ROR
        XORI #9,R0
        MVO R0,_rand

        ; Increase frame number
        MVI _frame,R0
        INCR R0
        MVO R0,_frame

        RETURN
        ENDP
    
        ORG $330,$330,"-RWB"
        ;
        ; 16-bits variables
        ;
_frame:     RMB 1       ; Current frame
_read:      RMB 1       ; Pointer to DATA
_gram_bitmap:   RMB 1   ; Bitmap for definition
_screen:    RMB 1       ; Pointer to current screen position
_color:     RMB 1       ; Current color
_mobs:      RMB 3*8     ; MOB buffer
_col0:      RMB 1       ; Collision status for MOB0
_col1:      RMB 1       ; Collision status for MOB1
_col2:      RMB 1       ; Collision status for MOB2
_col3:      RMB 1       ; Collision status for MOB3
_col4:      RMB 1       ; Collision status for MOB4
_col5:      RMB 1       ; Collision status for MOB5
_col6:      RMB 1       ; Collision status for MOB6
_col7:      RMB 1       ; Collision status for MOB7

SCRATCH:    ORG $100,$100,"-RWBN"
        ;
        ; 8-bits variables
        ;
ISRVEC:     RMB 2       ; Pointer to ISR vector (require by Intellivision ROM)
_int:       RMB 1       ; Signals interrupt received
_rand:      RMB 1       ; Pseudo-random value
_load_gram: RMB 1       ; Signals GRAM definition waiting
_gram_target:   RMB 1   ; Contains GRAM card number
_gram_total:    RMB 1   ; Contains total GRAM cards for definition
_first_time:    RMB 1   ; Signals program started for first time
